// CkImap.h: interface for the CkImap class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _CKIMAP_H
#define _CKIMAP_H

#pragma once

class CkByteData;
class CkString;
class CkMailboxes;
class CkMessageSet;
class CkEmail;
class CkEmailBundle;
class CkStringArray;
#include "CkObject.h"

/*
    IMPORTANT: Objects returned by methods as non-const pointers must be deleted
    by the calling application. 

  */

// CLASS: CkImap
class CkImap  : public CkObject
{
    public:
	CkImap();
	virtual ~CkImap();

	// BEGIN PUBLIC INTERFACE
	bool get_Utf8(void) const;
	void put_Utf8(bool b);


	bool Connect(const char *hostname);
	bool Disconnect(void);
	bool IsConnected(void);

	bool Login(const char *login, const char *password);
	bool Logout(void);
	bool IsLoggedIn(void);

	void get_LoggedInUser(CkString &str);
	void get_ConnectedToHost(CkString &str);

	bool CreateMailbox(const char *mailbox);
	bool DeleteMailbox(const char *mailbox);
	bool RenameMailbox(const char *fromMailbox, const char *toMailbox);

	// Gets the last response returned by the IMAP server.
	void LastResponse(CkString &str);

	CkMailboxes *ListMailboxes(const char *reference, const char *wildcardedMailbox);

	bool SelectMailbox(const char *mailbox);    // Email flags are modified when full email is fetched.
	bool ExamineMailbox(const char *mailbox);   // Email flags are not modified, mailbox cannot be modified.

	// Search for messages and return a message set of either UIDs (if bUid = true) or sequence numbers.
	CkMessageSet *Search(const char *criteria, bool bUid);

	CkMessageSet *GetAllUids(void);

	CkEmail *FetchSingle(long msgId, bool bUid);
	CkEmail *FetchSingleHeader(long msgId, bool bUid);
	CkEmailBundle *FetchBundle(CkMessageSet &messageSet);
	CkEmailBundle *FetchHeaders(CkMessageSet &messageSet);
	CkEmailBundle *FetchChunk(int startSeqNum, int count, CkMessageSet &failedSet, CkMessageSet &fetchedSet);

	bool FetchSingleAsMime(long msgId, bool bUid, CkString &strMime);
	CkStringArray *FetchBundleAsMime(CkMessageSet &messageSet);

	bool Expunge(void);
	bool ExpungeAndClose(void);

	bool StoreFlags(long msgId, bool bUid, const char *flagName, long value);
	bool FetchFlags(long msgId, bool bUid, CkString &strFlags);

	bool SetFlag(long msgId, bool bUid, const char *flagName, long value);
	bool SetFlags(CkMessageSet &messageSet, const char *flagName, long value);

	void get_AuthMethod(CkString &str);
	void put_AuthMethod(const char *str);

	void get_Domain(CkString &str);
	void put_Domain(const char *str);

	int get_Port(void);
	void put_Port(int port);

	bool get_Ssl(void);
	void put_Ssl(bool s);

	bool AppendMime(const char *mailbox, const char *mimeText);
	bool AppendMail(const char *mailbox, CkEmail &email);

	// Copy a message from the currently selected mailbox to a new mailbox.
	bool Copy(long msgId, bool bUid, const char *copyToMailbox);

	bool SetMailFlag(CkEmail &email, const char *flagName, long value);
	long GetMailFlag(CkEmail &email, const char *flagName);
	// Query the IMAP server get get up-to-date flag information.
	bool RefetchMailFlags(CkEmail &email);

	// When headers are downloaded, information about the email can be retrieved
	// using these methods.
	long GetMailSize(CkEmail &email);
	long GetMailNumAttach(CkEmail &email);
	long GetMailAttachSize(CkEmail &email, long attachIndex);
	bool GetMailAttachFilename(CkEmail &email, long attachIndex, CkString &strFilename);

	// Automatically decrypt and/or verify signatures (the default).
	// Results of the "unwrapping" are available via the CkEmail object.
	bool get_AutoUnwrapSecurity(void);
	void put_AutoUnwrapSecurity(bool bUnwrap);

	// When peek-mode is true, fetching entire emails will not affect
	// the email flags, regardless of whether the mailbox was "selected"
	// using SelectMailbox or "examined" with ExamineMailbox.
	bool get_PeekMode(void);
	void put_PeekMode(bool bPeek);

	// If AppendSeen is true (the default), then AppendMail and AppendMime will append
	// an email as already-seen, otherwise it will be initialized to "unseen".
	bool get_AppendSeen(void);
	void put_AppendSeen(bool alreadySeen);

	// Error log retrieval and saving (these methods are common to all Chilkat VC++ classes.)
	bool SaveLastError(const char *filename);
        void LastErrorXml(CkString &str);
        void LastErrorHtml(CkString &str);
        void LastErrorText(CkString &str);


	bool UnlockComponent(const char *unlockCode);
	bool IsUnlocked(void) const;


	// timeouts in seconds...
	int get_ReadTimeout(void) const;
	void put_ReadTimeout(int numSec);

	int get_ConnectTimeout(void) const;
	void put_ConnectTimeout(int numSec);


	bool SendRawCommand(const char *rawCommand, CkString &rawResponse);

	// END PUBLIC INTERFACE


    // For internal use only.
    private:
	void *m_impl;
	bool m_utf8;	// If true, all input "const char *" parameters are utf-8, otherwise they are ANSI strings.

	// Don't allow assignment or copying these objects.
	CkImap(const CkImap &) { } 
	CkImap &operator=(const CkImap &) { return *this; }

    public:
	void *getImpl(void) { return m_impl; } 

	CkImap(void *impl) : m_impl(impl) { }


};


#endif


